require 'colorize'
require 'fileutils'

module QuiversTaskrunner
	module_function

	def create_azure_files_restore_points(files_to_backup)

		# convert single file into an array
		files = files_to_backup.is_a?(Array) ? files_to_backup : (files_to_backup.nil? ? [] : [files_to_backup])

		files.each do |f|
			f_ext = File.extname(f)
			f_backup_ext = ".backup#{f_ext}"
			f_backup = f.gsub(f_ext, f_backup_ext)
			# 1. Delete previous backup files if there are still present
			FileUtils.rm_f(f_backup) if FileTest.exists? f_backup
			# 2. Create backup files
			FileUtils.cp f, f_backup
		end
	end

	def restore_and_clean_files(files_to_restore)

		# convert single file into an array
		files = files_to_restore.is_a?(Array) ? files_to_restore : (files_to_restore.nil? ? [] : [files_to_restore])

		files.each do |f|
			f_ext = File.extname(f)
			f_backup_ext = ".backup#{f_ext}"
			f_backup = f.gsub(f_ext, f_backup_ext)
			# 1. Make sure there are restore files available
			raise StandardError, "Missing restore file #{f_backup}"  unless FileTest.exists? f_backup
			# 2. Delete the modified files
			FileUtils.rm_f(f)
			# 3. Restore modified files
			FileUtils.cp f_backup, f
			# 4. Delete backup files
			FileUtils.rm_f(f_backup)
		end
	end
end